#!/usr/bin/env python

#   The gedit XML Tools plugin provides many useful tools for xml development.
#   Copyright (C) 2008  Simon Wenner
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gettext import gettext as _

import gtk
import gedit
from xmlprocessor import XMLToolsXmlProcessor

# Tools menu items
ui_str = """<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="ToolsOps_2">
        <menuitem name="XMLTools" action="XMLvalidate"/>
        <menuitem name="XMLTools1" action="XMLxpath"/>
        <!-- <menuitem name="XMLTools2" action="XMLxquery"/> -->
      </placeholder>
    </menu>
  </menubar>
</ui>
"""

class XMLToolsWindowHelper:
	def __init__(self, plugin, window):
		self._window = window
		self._plugin = plugin
		
		# add bottom panel field
		self._scroll_field = gtk.ScrolledWindow()
		self._panel_field = gtk.TextView()
		self._output_buffer = self._panel_field.get_buffer()
		self._scroll_field.add_with_viewport(self._panel_field)
		
		# set properties of panel field
		self._panel_field.set_editable(False)
		self._scroll_field.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self._scroll_field.show_all()
		
		panel = window.get_bottom_panel()
		panel.add_item(self._scroll_field, "XML Tools", "gtk-properties")

		# Insert menu items
		self._insert_menu()

	def deactivate(self):
		# Remove any installed menu items
		self._remove_menu()

		self._window = None
		self._plugin = None
		self._action_group = None
		
		panel = window.get_bottom_panel()
		panel.remove_item(self._scroll_field)
		self._scroll_field = None

	def _insert_menu(self):
		# Get the GtkUIManager
		manager = self._window.get_ui_manager()

		# Create a new action group
		self._action_group = gtk.ActionGroup("XMLToolsPyPluginActions")
		self._action_group.add_actions([("XMLvalidate", None, _("XML validation"),
										'F5', _("validate an XML file"),
										self.validate_document)])
		self._action_group.add_actions([("XMLxpath", None, _("XPath query ..."),
										None, _("XPath query editor"),
										self.create_xpath_query_editor)])
		#self._action_group.add_actions([("XMLxquery", None, _("XQuery ..."),
		#								None, _("XQuery editor"),
		#								self.xquery_on_document)])

		# Insert the action group
		manager.insert_action_group(self._action_group, -1)

		# Merge the UI
		self._ui_id = manager.add_ui_from_string(ui_str)

	def _remove_menu(self):
		# Get the GtkUIManager
		manager = self._window.get_ui_manager()

		# Remove the ui
		manager.remove_ui(self._ui_id)

		# Remove the action group
		manager.remove_action_group(self._action_group)

		# Make sure the manager updates
		manager.ensure_update()

	def update_ui(self):
		self._action_group.set_sensitive(self._window.get_active_document() != None)

	# Menu activate handlers
	def validate_document(self, action):
		doc = self._window.get_active_document()
		if not doc:
			return
		
		uri = doc.get_uri_for_display() 
		buff = "Validating: " + uri + "\n"
		xml = XMLToolsXmlProcessor(uri)
		if xml.valid:
			buff += "Document is valid!"
		else:
			buff += "Document is NOT valid!\n"
			for t in xml.get_error():
				buff += "Error on line: " + str(t[0]) + " -- " + t[1] + "\n"
		
		self._output_buffer.set_text(buff)
		panel = self._window.get_bottom_panel()
		panel.activate_item(self._scroll_field)
		panel.set_property("visible", True)
		

	def create_xpath_query_editor(self, action):
		qwin = XMLQueryWindow(self)
		
	
	def xpath_query_on_document(self, string):
		doc = self._window.get_active_document()
		if not doc:
			return
		
		# do query
		xml = XMLToolsXmlProcessor(doc.get_uri_for_display())
		res = xml.xpath(string)
		# TODO: error handling
		
		# show result
		self._output_buffer.set_text("Xpath result:\n" + res)
		panel = self._window.get_bottom_panel()
		panel.activate_item(self._scroll_field)
		panel.set_property("visible", True)
		

	def xquery_on_document(self, action):
		doc = self._window.get_active_document()
		if not doc:
			return

		print "STUB: XQuery!"

class XMLQueryWindow:
	def __init__(self, window_helper):
		self._window_helper = window_helper

		self._window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self._window.set_title("Xpath query editor")
		self._window.set_border_width(10)
		self._window.set_position(gtk.WIN_POS_CENTER)
		#self._window.set_resizable(False)
		
		self._window.connect("delete_event", self.delete_event)
		#window.connect("destroy", self.destroy)
		
		self.field = gtk.ScrolledWindow()
		self.tv = gtk.TextView()
		self.field.add_with_viewport(self.tv)
		self.field.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.field.set_size_request(400, 200)
		
		vbox = gtk.VBox(False, 10)
		vbox.pack_start(self.field, False, False, 0)
		hbox = gtk.HBox(False, 0)
		
		button = gtk.Button(None, gtk.STOCK_EXECUTE)
		button.connect("clicked", self.query_event, None)
		hbox.pack_start(button, False, False, 0)
		
		button = gtk.Button(None, gtk.STOCK_CLEAR)
		button.connect("clicked", self.clear_event, None)
		hbox.pack_start(button, False, False, 0)
		
		button = gtk.Button(None, gtk.STOCK_CLOSE)
		button.connect("clicked", self.delete_event, None)
		hbox.pack_end(button, False, False, 0)
		
		vbox.pack_start(hbox, False, False, 0)	
		self._window.add(vbox)
	
		self._window.show_all()

	def delete_event(self, widget, event, data=None):
		self._window.destroy()
		return False
		
	def query_event(self, widget, event, data=None):
		buff = self.tv.get_buffer()
		self._window_helper.xpath_query_on_document(buff.get_text(buff.get_start_iter(), buff.get_end_iter(), True))
		return False
		
	def clear_event(self, widget, event, data=None):
		self.tv.set_buffer(gtk.TextBuffer(None))
		return False



class XMLTools(gedit.Plugin):
	def __init__(self):
		gedit.Plugin.__init__(self)
		self._instances = {}

	def activate(self, window):
		self._instances[window] = XMLToolsWindowHelper(self, window)

	def deactivate(self, window):
		self._instances[window].deactivate()
		del self._instances[window]

	def update_ui(self, window):
		self._instances[window].update_ui()

